# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.tenant_custom_nameservers import tenant_custom_nameserver_create_params
from ...types.tenant_custom_nameservers.tenant_custom_nameserver_get_response import TenantCustomNameserverGetResponse
from ...types.tenant_custom_nameservers.tenant_custom_nameserver_create_response import (
    TenantCustomNameserverCreateResponse,
)
from ...types.tenant_custom_nameservers.tenant_custom_nameserver_delete_response import (
    TenantCustomNameserverDeleteResponse,
)

__all__ = ["TenantCustomNameserversResource", "AsyncTenantCustomNameserversResource"]


class TenantCustomNameserversResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TenantCustomNameserversResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TenantCustomNameserversResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantCustomNameserversResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TenantCustomNameserversResourceWithStreamingResponse(self)

    def create(
        self,
        tenant_tag: str,
        *,
        ns_name: str,
        ns_set: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TenantCustomNameserverCreateResponse]:
        """
        Add Tenant Custom Nameserver

        Args:
          tenant_tag: Tenant identifier tag.

          ns_name: The FQDN of the name server.

          ns_set: The number of the set that this name server belongs to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_tag:
            raise ValueError(f"Expected a non-empty value for `tenant_tag` but received {tenant_tag!r}")
        return self._post(
            path_template("/tenants/{tenant_tag}/custom_ns", tenant_tag=tenant_tag),
            body=maybe_transform(
                {
                    "ns_name": ns_name,
                    "ns_set": ns_set,
                },
                tenant_custom_nameserver_create_params.TenantCustomNameserverCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TenantCustomNameserverCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TenantCustomNameserverCreateResponse]],
                ResultWrapper[TenantCustomNameserverCreateResponse],
            ),
        )

    def delete(
        self,
        custom_ns_id: str,
        *,
        tenant_tag: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TenantCustomNameserverDeleteResponse]:
        """
        Delete Tenant Custom Nameserver

        Args:
          tenant_tag: Tenant identifier tag.

          custom_ns_id: The FQDN of the name server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_tag:
            raise ValueError(f"Expected a non-empty value for `tenant_tag` but received {tenant_tag!r}")
        if not custom_ns_id:
            raise ValueError(f"Expected a non-empty value for `custom_ns_id` but received {custom_ns_id!r}")
        return self._get_api_list(
            path_template(
                "/tenants/{tenant_tag}/custom_ns/{custom_ns_id}", tenant_tag=tenant_tag, custom_ns_id=custom_ns_id
            ),
            page=SyncSinglePage[TenantCustomNameserverDeleteResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
            method="delete",
        )

    def get(
        self,
        tenant_tag: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[TenantCustomNameserverGetResponse]:
        """
        List a tenant's custom nameservers.

        Args:
          tenant_tag: Tenant identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_tag:
            raise ValueError(f"Expected a non-empty value for `tenant_tag` but received {tenant_tag!r}")
        return self._get_api_list(
            path_template("/tenants/{tenant_tag}/custom_ns", tenant_tag=tenant_tag),
            page=SyncSinglePage[TenantCustomNameserverGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TenantCustomNameserverGetResponse,
        )


class AsyncTenantCustomNameserversResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTenantCustomNameserversResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantCustomNameserversResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantCustomNameserversResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTenantCustomNameserversResourceWithStreamingResponse(self)

    async def create(
        self,
        tenant_tag: str,
        *,
        ns_name: str,
        ns_set: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[TenantCustomNameserverCreateResponse]:
        """
        Add Tenant Custom Nameserver

        Args:
          tenant_tag: Tenant identifier tag.

          ns_name: The FQDN of the name server.

          ns_set: The number of the set that this name server belongs to.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_tag:
            raise ValueError(f"Expected a non-empty value for `tenant_tag` but received {tenant_tag!r}")
        return await self._post(
            path_template("/tenants/{tenant_tag}/custom_ns", tenant_tag=tenant_tag),
            body=await async_maybe_transform(
                {
                    "ns_name": ns_name,
                    "ns_set": ns_set,
                },
                tenant_custom_nameserver_create_params.TenantCustomNameserverCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[TenantCustomNameserverCreateResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[TenantCustomNameserverCreateResponse]],
                ResultWrapper[TenantCustomNameserverCreateResponse],
            ),
        )

    def delete(
        self,
        custom_ns_id: str,
        *,
        tenant_tag: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TenantCustomNameserverDeleteResponse, AsyncSinglePage[TenantCustomNameserverDeleteResponse]]:
        """
        Delete Tenant Custom Nameserver

        Args:
          tenant_tag: Tenant identifier tag.

          custom_ns_id: The FQDN of the name server.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_tag:
            raise ValueError(f"Expected a non-empty value for `tenant_tag` but received {tenant_tag!r}")
        if not custom_ns_id:
            raise ValueError(f"Expected a non-empty value for `custom_ns_id` but received {custom_ns_id!r}")
        return self._get_api_list(
            path_template(
                "/tenants/{tenant_tag}/custom_ns/{custom_ns_id}", tenant_tag=tenant_tag, custom_ns_id=custom_ns_id
            ),
            page=AsyncSinglePage[TenantCustomNameserverDeleteResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=str,
            method="delete",
        )

    def get(
        self,
        tenant_tag: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TenantCustomNameserverGetResponse, AsyncSinglePage[TenantCustomNameserverGetResponse]]:
        """
        List a tenant's custom nameservers.

        Args:
          tenant_tag: Tenant identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_tag:
            raise ValueError(f"Expected a non-empty value for `tenant_tag` but received {tenant_tag!r}")
        return self._get_api_list(
            path_template("/tenants/{tenant_tag}/custom_ns", tenant_tag=tenant_tag),
            page=AsyncSinglePage[TenantCustomNameserverGetResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=TenantCustomNameserverGetResponse,
        )


class TenantCustomNameserversResourceWithRawResponse:
    def __init__(self, tenant_custom_nameservers: TenantCustomNameserversResource) -> None:
        self._tenant_custom_nameservers = tenant_custom_nameservers

        self.create = to_raw_response_wrapper(
            tenant_custom_nameservers.create,
        )
        self.delete = to_raw_response_wrapper(
            tenant_custom_nameservers.delete,
        )
        self.get = to_raw_response_wrapper(
            tenant_custom_nameservers.get,
        )


class AsyncTenantCustomNameserversResourceWithRawResponse:
    def __init__(self, tenant_custom_nameservers: AsyncTenantCustomNameserversResource) -> None:
        self._tenant_custom_nameservers = tenant_custom_nameservers

        self.create = async_to_raw_response_wrapper(
            tenant_custom_nameservers.create,
        )
        self.delete = async_to_raw_response_wrapper(
            tenant_custom_nameservers.delete,
        )
        self.get = async_to_raw_response_wrapper(
            tenant_custom_nameservers.get,
        )


class TenantCustomNameserversResourceWithStreamingResponse:
    def __init__(self, tenant_custom_nameservers: TenantCustomNameserversResource) -> None:
        self._tenant_custom_nameservers = tenant_custom_nameservers

        self.create = to_streamed_response_wrapper(
            tenant_custom_nameservers.create,
        )
        self.delete = to_streamed_response_wrapper(
            tenant_custom_nameservers.delete,
        )
        self.get = to_streamed_response_wrapper(
            tenant_custom_nameservers.get,
        )


class AsyncTenantCustomNameserversResourceWithStreamingResponse:
    def __init__(self, tenant_custom_nameservers: AsyncTenantCustomNameserversResource) -> None:
        self._tenant_custom_nameservers = tenant_custom_nameservers

        self.create = async_to_streamed_response_wrapper(
            tenant_custom_nameservers.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            tenant_custom_nameservers.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            tenant_custom_nameservers.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SchemaDataParam"]


class SchemaDataParam(TypedDict, total=False):
    """
    The configuration object which contains the details for the WARP client to conduct the test.
    """

    host: Required[str]
    """The desired endpoint to test."""

    kind: Required[Literal["http", "traceroute"]]
    """The type of test."""

    method: Literal["GET"]
    """The HTTP request method type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePagination, AsyncV4PagePagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.dex import rule_list_params, rule_create_params, rule_update_params
from ....types.zero_trust.dex.rule_get_response import RuleGetResponse
from ....types.zero_trust.dex.rule_list_response import RuleListResponse
from ....types.zero_trust.dex.rule_create_response import RuleCreateResponse
from ....types.zero_trust.dex.rule_delete_response import RuleDeleteResponse
from ....types.zero_trust.dex.rule_update_response import RuleUpdateResponse

__all__ = ["RulesResource", "AsyncRulesResource"]


class RulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        match: str,
        name: str,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleCreateResponse]:
        """
        Create a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          match: The wirefilter expression to match.

          name: The name of the Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dex/rules", account_id=account_id),
            body=maybe_transform(
                {
                    "match": match,
                    "name": name,
                    "description": description,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleCreateResponse]], ResultWrapper[RuleCreateResponse]),
        )

    def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        match: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleUpdateResponse]:
        """
        Update a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          rule_id: API Resource UUID tag.

          match: The wirefilter expression to match.

          name: The name of the Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/dex/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=maybe_transform(
                {
                    "description": description,
                    "match": match,
                    "name": name,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleUpdateResponse]], ResultWrapper[RuleUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float,
        per_page: float,
        name: str | Omit = omit,
        sort_by: Literal["name", "created_at", "updated_at"] | Omit = omit,
        sort_order: Literal["ASC", "DESC"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[Optional[RuleListResponse]]:
        """
        List DEX Rules.

        Args:
          account_id: Unique identifier linked to an account.

          page: Page number of paginated results.

          per_page: Number of results per page.

          name: Filter results by rule name.

          sort_by: Which property to sort results by.

          sort_order: Sort direction for sort_by property.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dex/rules", account_id=account_id),
            page=SyncV4PagePagination[Optional[RuleListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "name": name,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    rule_list_params.RuleListParams,
                ),
            ),
            model=RuleListResponse,
        )

    def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleDeleteResponse]:
        """
        Delete a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          rule_id: API Resource UUID tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/dex/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleDeleteResponse]], ResultWrapper[RuleDeleteResponse]),
        )

    def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleGetResponse]:
        """
        Get details for a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          rule_id: API Resource UUID tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/dex/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleGetResponse]], ResultWrapper[RuleGetResponse]),
        )


class AsyncRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        match: str,
        name: str,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleCreateResponse]:
        """
        Create a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          match: The wirefilter expression to match.

          name: The name of the Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dex/rules", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "match": match,
                    "name": name,
                    "description": description,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleCreateResponse]], ResultWrapper[RuleCreateResponse]),
        )

    async def update(
        self,
        rule_id: str,
        *,
        account_id: str,
        description: str | Omit = omit,
        match: str | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleUpdateResponse]:
        """
        Update a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          rule_id: API Resource UUID tag.

          match: The wirefilter expression to match.

          name: The name of the Rule.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/dex/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=await async_maybe_transform(
                {
                    "description": description,
                    "match": match,
                    "name": name,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleUpdateResponse]], ResultWrapper[RuleUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float,
        per_page: float,
        name: str | Omit = omit,
        sort_by: Literal["name", "created_at", "updated_at"] | Omit = omit,
        sort_order: Literal["ASC", "DESC"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[RuleListResponse], AsyncV4PagePagination[Optional[RuleListResponse]]]:
        """
        List DEX Rules.

        Args:
          account_id: Unique identifier linked to an account.

          page: Page number of paginated results.

          per_page: Number of results per page.

          name: Filter results by rule name.

          sort_by: Which property to sort results by.

          sort_order: Sort direction for sort_by property.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dex/rules", account_id=account_id),
            page=AsyncV4PagePagination[Optional[RuleListResponse]],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "name": name,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    rule_list_params.RuleListParams,
                ),
            ),
            model=RuleListResponse,
        )

    async def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleDeleteResponse]:
        """
        Delete a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          rule_id: API Resource UUID tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/dex/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleDeleteResponse]], ResultWrapper[RuleDeleteResponse]),
        )

    async def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[RuleGetResponse]:
        """
        Get details for a DEX Rule.

        Args:
          account_id: Unique identifier linked to an account.

          rule_id: API Resource UUID tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/dex/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[RuleGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[RuleGetResponse]], ResultWrapper[RuleGetResponse]),
        )


class RulesResourceWithRawResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_raw_response_wrapper(
            rules.create,
        )
        self.update = to_raw_response_wrapper(
            rules.update,
        )
        self.list = to_raw_response_wrapper(
            rules.list,
        )
        self.delete = to_raw_response_wrapper(
            rules.delete,
        )
        self.get = to_raw_response_wrapper(
            rules.get,
        )


class AsyncRulesResourceWithRawResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_raw_response_wrapper(
            rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rules.delete,
        )
        self.get = async_to_raw_response_wrapper(
            rules.get,
        )


class RulesResourceWithStreamingResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_streamed_response_wrapper(
            rules.create,
        )
        self.update = to_streamed_response_wrapper(
            rules.update,
        )
        self.list = to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            rules.delete,
        )
        self.get = to_streamed_response_wrapper(
            rules.get,
        )


class AsyncRulesResourceWithStreamingResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_streamed_response_wrapper(
            rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rules.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            rules.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["ScheduleUpdateParams", "Body"]


class ScheduleUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    body: Required[Iterable[Body]]


class Body(TypedDict, total=False):
    cron: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .....pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ....._base_client import AsyncPaginator, make_request_options
from .....types.email_security.investigate.bulk import message_list_params
from .....types.email_security.investigate.bulk.message_list_response import MessageListResponse

__all__ = ["MessagesResource", "AsyncMessagesResource"]


class MessagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return MessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return MessagesResourceWithStreamingResponse(self)

    def list(
        self,
        job_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["PENDING", "DISCOVERING", "PROCESSING", "COMPLETED", "FAILED", "CANCELLED", "SKIPPED"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[MessageListResponse]:
        """
        List messages for a bulk action job

        Args:
          account_id: Identifier.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/email-security/investigate/bulk/{job_id}/messages",
                account_id=account_id,
                job_id=job_id,
            ),
            page=SyncV4PagePaginationArray[MessageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    message_list_params.MessageListParams,
                ),
            ),
            model=MessageListResponse,
        )


class AsyncMessagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncMessagesResourceWithStreamingResponse(self)

    def list(
        self,
        job_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        status: Literal["PENDING", "DISCOVERING", "PROCESSING", "COMPLETED", "FAILED", "CANCELLED", "SKIPPED"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[MessageListResponse, AsyncV4PagePaginationArray[MessageListResponse]]:
        """
        List messages for a bulk action job

        Args:
          account_id: Identifier.

          page: Current page within paginated list of results.

          per_page: The number of results per page. Maximum value is 1000.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/email-security/investigate/bulk/{job_id}/messages",
                account_id=account_id,
                job_id=job_id,
            ),
            page=AsyncV4PagePaginationArray[MessageListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "status": status,
                    },
                    message_list_params.MessageListParams,
                ),
            ),
            model=MessageListResponse,
        )


class MessagesResourceWithRawResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.list = to_raw_response_wrapper(
            messages.list,
        )


class AsyncMessagesResourceWithRawResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.list = async_to_raw_response_wrapper(
            messages.list,
        )


class MessagesResourceWithStreamingResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.list = to_streamed_response_wrapper(
            messages.list,
        )


class AsyncMessagesResourceWithStreamingResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.list = async_to_streamed_response_wrapper(
            messages.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PrefixBindingListParams"]


class PrefixBindingListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier of a Cloudflare account."""

    cursor: str
    """Opaque token for cursor-based pagination.

    Omit for the first page. Pass the value from a previous response to fetch the
    next page.
    """

    per_page: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ct import (
    CTResource,
    AsyncCTResource,
    CTResourceWithRawResponse,
    AsyncCTResourceWithRawResponse,
    CTResourceWithStreamingResponse,
    AsyncCTResourceWithStreamingResponse,
)
from .alerting import (
    AlertingResource,
    AsyncAlertingResource,
    AlertingResourceWithRawResponse,
    AsyncAlertingResourceWithRawResponse,
    AlertingResourceWithStreamingResponse,
    AsyncAlertingResourceWithStreamingResponse,
)

__all__ = [
    "AlertingResource",
    "AsyncAlertingResource",
    "AlertingResourceWithRawResponse",
    "AsyncAlertingResourceWithRawResponse",
    "AlertingResourceWithStreamingResponse",
    "AsyncAlertingResourceWithStreamingResponse",
    "CTResource",
    "AsyncCTResource",
    "CTResourceWithRawResponse",
    "AsyncCTResourceWithRawResponse",
    "CTResourceWithStreamingResponse",
    "AsyncCTResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from .setup_flows import (
    SetupFlowsResource,
    AsyncSetupFlowsResource,
    SetupFlowsResourceWithRawResponse,
    AsyncSetupFlowsResourceWithRawResponse,
    SetupFlowsResourceWithStreamingResponse,
    AsyncSetupFlowsResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.zero_trust.casb import application_list_params
from .....types.zero_trust.casb.application_get_response import ApplicationGetResponse
from .....types.zero_trust.casb.application_list_response import ApplicationListResponse

__all__ = ["ApplicationsResource", "AsyncApplicationsResource"]


class ApplicationsResource(SyncAPIResource):
    @cached_property
    def setup_flows(self) -> SetupFlowsResource:
        return SetupFlowsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ApplicationsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        environment: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationListResponse:
        """
        Returns a list of available applications with use cases and permissions.

        Args:
          environment: Filter by supported environment (standard, fedramp).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/one/applications", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"environment": environment}, application_list_params.ApplicationListParams),
            ),
            cast_to=ApplicationListResponse,
        )

    def get(
        self,
        slug: Literal["GITHUB", "GOOGLE_WORKSPACE", "MICROSOFT_INTERNAL", "SALESFORCE", "SLACK"],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationGetResponse:
        """
        Returns full application details including auth methods, use cases, and
        permissions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return self._get(
            path_template("/accounts/{account_id}/one/applications/{slug}", account_id=account_id, slug=slug),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationGetResponse,
        )


class AsyncApplicationsResource(AsyncAPIResource):
    @cached_property
    def setup_flows(self) -> AsyncSetupFlowsResource:
        return AsyncSetupFlowsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncApplicationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncApplicationsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        account_id: str,
        environment: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationListResponse:
        """
        Returns a list of available applications with use cases and permissions.

        Args:
          environment: Filter by supported environment (standard, fedramp).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/one/applications", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"environment": environment}, application_list_params.ApplicationListParams
                ),
            ),
            cast_to=ApplicationListResponse,
        )

    async def get(
        self,
        slug: Literal["GITHUB", "GOOGLE_WORKSPACE", "MICROSOFT_INTERNAL", "SALESFORCE", "SLACK"],
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ApplicationGetResponse:
        """
        Returns full application details including auth methods, use cases, and
        permissions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return await self._get(
            path_template("/accounts/{account_id}/one/applications/{slug}", account_id=account_id, slug=slug),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ApplicationGetResponse,
        )


class ApplicationsResourceWithRawResponse:
    def __init__(self, applications: ApplicationsResource) -> None:
        self._applications = applications

        self.list = to_raw_response_wrapper(
            applications.list,
        )
        self.get = to_raw_response_wrapper(
            applications.get,
        )

    @cached_property
    def setup_flows(self) -> SetupFlowsResourceWithRawResponse:
        return SetupFlowsResourceWithRawResponse(self._applications.setup_flows)


class AsyncApplicationsResourceWithRawResponse:
    def __init__(self, applications: AsyncApplicationsResource) -> None:
        self._applications = applications

        self.list = async_to_raw_response_wrapper(
            applications.list,
        )
        self.get = async_to_raw_response_wrapper(
            applications.get,
        )

    @cached_property
    def setup_flows(self) -> AsyncSetupFlowsResourceWithRawResponse:
        return AsyncSetupFlowsResourceWithRawResponse(self._applications.setup_flows)


class ApplicationsResourceWithStreamingResponse:
    def __init__(self, applications: ApplicationsResource) -> None:
        self._applications = applications

        self.list = to_streamed_response_wrapper(
            applications.list,
        )
        self.get = to_streamed_response_wrapper(
            applications.get,
        )

    @cached_property
    def setup_flows(self) -> SetupFlowsResourceWithStreamingResponse:
        return SetupFlowsResourceWithStreamingResponse(self._applications.setup_flows)


class AsyncApplicationsResourceWithStreamingResponse:
    def __init__(self, applications: AsyncApplicationsResource) -> None:
        self._applications = applications

        self.list = async_to_streamed_response_wrapper(
            applications.list,
        )
        self.get = async_to_streamed_response_wrapper(
            applications.get,
        )

    @cached_property
    def setup_flows(self) -> AsyncSetupFlowsResourceWithStreamingResponse:
        return AsyncSetupFlowsResourceWithStreamingResponse(self._applications.setup_flows)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.tunnels.warp_connector import configuration_update_params
from .....types.zero_trust.tunnels.warp_connector.configuration_get_response import ConfigurationGetResponse
from .....types.zero_trust.tunnels.warp_connector.configuration_update_response import ConfigurationUpdateResponse

__all__ = ["ConfigurationsResource", "AsyncConfigurationsResource"]


class ConfigurationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConfigurationsResourceWithStreamingResponse(self)

    def update(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        ha_mode: Literal["none", "disabled", "aws", "local"],
        config: Optional[configuration_update_params.Config] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationUpdateResponse]:
        """
        Adds or updates the high-availability configuration for a WARP Connector tunnel.

        Args:
          account_id: Identifier.

          tunnel_id: UUID of the tunnel.

          ha_mode: High-availability mode for the WARP Connector tunnel. `none` means HA is enabled
              but no provider is configured yet (newly created tunnels default to this).
              `disabled` means HA is explicitly turned off. `aws` uses AWS ENI move for
              failover. `local` uses virtual IPs (VIPs) on the local interface.

          config: Provider-specific configuration. Required shape depends on ha_mode. For `aws`,
              must contain `fnr_id`. For `local`, must contain `vips`. For `none` and
              `disabled`, must be empty or omitted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}/configurations",
                account_id=account_id,
                tunnel_id=tunnel_id,
            ),
            body=maybe_transform(
                {
                    "ha_mode": ha_mode,
                    "config": config,
                },
                configuration_update_params.ConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationUpdateResponse]], ResultWrapper[ConfigurationUpdateResponse]),
        )

    def get(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationGetResponse]:
        """
        Gets the high-availability configuration for a WARP Connector tunnel.

        Args:
          account_id: Identifier.

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}/configurations",
                account_id=account_id,
                tunnel_id=tunnel_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationGetResponse]], ResultWrapper[ConfigurationGetResponse]),
        )


class AsyncConfigurationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConfigurationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigurationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigurationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConfigurationsResourceWithStreamingResponse(self)

    async def update(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        ha_mode: Literal["none", "disabled", "aws", "local"],
        config: Optional[configuration_update_params.Config] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationUpdateResponse]:
        """
        Adds or updates the high-availability configuration for a WARP Connector tunnel.

        Args:
          account_id: Identifier.

          tunnel_id: UUID of the tunnel.

          ha_mode: High-availability mode for the WARP Connector tunnel. `none` means HA is enabled
              but no provider is configured yet (newly created tunnels default to this).
              `disabled` means HA is explicitly turned off. `aws` uses AWS ENI move for
              failover. `local` uses virtual IPs (VIPs) on the local interface.

          config: Provider-specific configuration. Required shape depends on ha_mode. For `aws`,
              must contain `fnr_id`. For `local`, must contain `vips`. For `none` and
              `disabled`, must be empty or omitted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}/configurations",
                account_id=account_id,
                tunnel_id=tunnel_id,
            ),
            body=await async_maybe_transform(
                {
                    "ha_mode": ha_mode,
                    "config": config,
                },
                configuration_update_params.ConfigurationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationUpdateResponse]], ResultWrapper[ConfigurationUpdateResponse]),
        )

    async def get(
        self,
        tunnel_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ConfigurationGetResponse]:
        """
        Gets the high-availability configuration for a WARP Connector tunnel.

        Args:
          account_id: Identifier.

          tunnel_id: UUID of the tunnel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not tunnel_id:
            raise ValueError(f"Expected a non-empty value for `tunnel_id` but received {tunnel_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/warp_connector/{tunnel_id}/configurations",
                account_id=account_id,
                tunnel_id=tunnel_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ConfigurationGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ConfigurationGetResponse]], ResultWrapper[ConfigurationGetResponse]),
        )


class ConfigurationsResourceWithRawResponse:
    def __init__(self, configurations: ConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = to_raw_response_wrapper(
            configurations.update,
        )
        self.get = to_raw_response_wrapper(
            configurations.get,
        )


class AsyncConfigurationsResourceWithRawResponse:
    def __init__(self, configurations: AsyncConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = async_to_raw_response_wrapper(
            configurations.update,
        )
        self.get = async_to_raw_response_wrapper(
            configurations.get,
        )


class ConfigurationsResourceWithStreamingResponse:
    def __init__(self, configurations: ConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = to_streamed_response_wrapper(
            configurations.update,
        )
        self.get = to_streamed_response_wrapper(
            configurations.get,
        )


class AsyncConfigurationsResourceWithStreamingResponse:
    def __init__(self, configurations: AsyncConfigurationsResource) -> None:
        self._configurations = configurations

        self.update = async_to_streamed_response_wrapper(
            configurations.update,
        )
        self.get = async_to_streamed_response_wrapper(
            configurations.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "SessionGetParticipantDataFromPeerIDResponse",
    "Data",
    "DataParticipant",
    "DataParticipantPeerEvent",
    "DataParticipantPeerReport",
    "DataParticipantPeerReportMetadata",
    "DataParticipantPeerReportMetadataAudioDevicesUpdate",
    "DataParticipantPeerReportMetadataAudioDevicesUpdateAdded",
    "DataParticipantPeerReportMetadataAudioDevicesUpdateRemoved",
    "DataParticipantPeerReportMetadataBrowserMetadata",
    "DataParticipantPeerReportMetadataCandidatePairs",
    "DataParticipantPeerReportMetadataCandidatePairsConsumingTransport",
    "DataParticipantPeerReportMetadataCandidatePairsProducingTransport",
    "DataParticipantPeerReportMetadataDeviceInfo",
    "DataParticipantPeerReportMetadataEvent",
    "DataParticipantPeerReportMetadataIPInformation",
    "DataParticipantPeerReportMetadataIPInformationASN",
    "DataParticipantPeerReportMetadataNativeMetadata",
    "DataParticipantPeerReportMetadataPcMetadata",
    "DataParticipantPeerReportMetadataSelectedDeviceUpdate",
    "DataParticipantPeerReportMetadataSelectedDeviceUpdateDevice",
    "DataParticipantPeerReportMetadataSpeakerDevicesUpdate",
    "DataParticipantPeerReportMetadataSpeakerDevicesUpdateAdded",
    "DataParticipantPeerReportMetadataSpeakerDevicesUpdateRemoved",
    "DataParticipantPeerReportMetadataVideoDevicesUpdate",
    "DataParticipantPeerReportMetadataVideoDevicesUpdateAdded",
    "DataParticipantPeerReportMetadataVideoDevicesUpdateRemoved",
    "DataParticipantPeerReportQuality",
    "DataParticipantPeerReportQualityAudioConsumer",
    "DataParticipantPeerReportQualityAudioConsumerCumulative",
    "DataParticipantPeerReportQualityAudioConsumerCumulativeJitterBufferDelay",
    "DataParticipantPeerReportQualityAudioConsumerCumulativePacketLoss",
    "DataParticipantPeerReportQualityAudioConsumerCumulativeQualityMos",
    "DataParticipantPeerReportQualityAudioProducer",
    "DataParticipantPeerReportQualityAudioProducerCumulative",
    "DataParticipantPeerReportQualityAudioProducerCumulativePacketLoss",
    "DataParticipantPeerReportQualityAudioProducerCumulativeQualityMos",
    "DataParticipantPeerReportQualityAudioProducerCumulativeRTT",
    "DataParticipantPeerReportQualityScreenshareAudioConsumer",
    "DataParticipantPeerReportQualityScreenshareAudioConsumerCumulative",
    "DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativeJitterBufferDelay",
    "DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativePacketLoss",
    "DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativeQualityMos",
    "DataParticipantPeerReportQualityScreenshareAudioProducer",
    "DataParticipantPeerReportQualityScreenshareAudioProducerCumulative",
    "DataParticipantPeerReportQualityScreenshareAudioProducerCumulativePacketLoss",
    "DataParticipantPeerReportQualityScreenshareAudioProducerCumulativeQualityMos",
    "DataParticipantPeerReportQualityScreenshareAudioProducerCumulativeRTT",
    "DataParticipantPeerReportQualityScreenshareVideoConsumer",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulative",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeFramePerSecond",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeFrameWidth",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeIssues",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeJitterBufferDelay",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativePacketLoss",
    "DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeQualityMos",
    "DataParticipantPeerReportQualityScreenshareVideoProducer",
    "DataParticipantPeerReportQualityScreenshareVideoProducerQualityLimitationDurations",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulative",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeFramePerSecond",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeFrameWidth",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeIssues",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulativePacketLoss",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeQualityMos",
    "DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeRTT",
    "DataParticipantPeerReportQualityVideoConsumer",
    "DataParticipantPeerReportQualityVideoConsumerCumulative",
    "DataParticipantPeerReportQualityVideoConsumerCumulativeFramePerSecond",
    "DataParticipantPeerReportQualityVideoConsumerCumulativeFrameWidth",
    "DataParticipantPeerReportQualityVideoConsumerCumulativeIssues",
    "DataParticipantPeerReportQualityVideoConsumerCumulativeJitterBufferDelay",
    "DataParticipantPeerReportQualityVideoConsumerCumulativePacketLoss",
    "DataParticipantPeerReportQualityVideoConsumerCumulativeQualityMos",
    "DataParticipantPeerReportQualityVideoProducer",
    "DataParticipantPeerReportQualityVideoProducerQualityLimitationDurations",
    "DataParticipantPeerReportQualityVideoProducerCumulative",
    "DataParticipantPeerReportQualityVideoProducerCumulativeFramePerSecond",
    "DataParticipantPeerReportQualityVideoProducerCumulativeFrameWidth",
    "DataParticipantPeerReportQualityVideoProducerCumulativeIssues",
    "DataParticipantPeerReportQualityVideoProducerCumulativePacketLoss",
    "DataParticipantPeerReportQualityVideoProducerCumulativeQualityMos",
    "DataParticipantPeerReportQualityVideoProducerCumulativeRTT",
]


class DataParticipantPeerEvent(BaseModel):
    """A connection lifecycle event recorded for a participant's peer."""

    id: Optional[str] = None
    """ID of the peer event."""

    created_at: Optional[str] = None
    """Timestamp when this peer event was created."""

    event_name: Optional[Literal["PEER_CREATED", "PEER_JOINING", "PEER_LEAVING"]] = None
    """Name of the peer event."""

    minutes_consumed: Optional[float] = None
    """Minutes consumed attributed to this event."""

    participant_id: Optional[str] = None
    """ID of the participant this event belongs to."""

    peer_id: Optional[str] = None
    """Peer ID this event belongs to."""

    preset_view_type: Optional[Literal["GROUP_CALL", "WEBINAR", "AUDIO_ROOM", "LIVESTREAM", "CHAT"]] = None
    """View type of the preset associated with the peer."""

    session_id: Optional[str] = None
    """ID of the session this event belongs to."""

    socket_session_id: Optional[str] = None
    """ID of the socket session associated with this event."""

    updated_at: Optional[str] = None
    """Timestamp when this peer event was last updated."""


class DataParticipantPeerReportMetadataAudioDevicesUpdateAdded(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataAudioDevicesUpdateRemoved(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataAudioDevicesUpdate(BaseModel):
    """A change to the set of available devices at a point in time."""

    added: Optional[List[DataParticipantPeerReportMetadataAudioDevicesUpdateAdded]] = None
    """Devices that became available."""

    removed: Optional[List[DataParticipantPeerReportMetadataAudioDevicesUpdateRemoved]] = None
    """Devices that became unavailable."""

    timestamp: Optional[str] = None
    """Timestamp of the device update."""


class DataParticipantPeerReportMetadataBrowserMetadata(BaseModel):
    browser: Optional[str] = None

    browser_version: Optional[str] = None

    engine: Optional[str] = None

    user_agent: Optional[str] = None

    webgl_support: Optional[bool] = None


class DataParticipantPeerReportMetadataCandidatePairsConsumingTransport(BaseModel):
    """ICE candidate pair statistics for a transport."""

    available_incoming_bitrate: Optional[float] = None

    available_outgoing_bitrate: Optional[float] = None

    bytes_discarded_on_send: Optional[float] = None

    bytes_received: Optional[float] = None

    bytes_sent: Optional[float] = None

    current_round_trip_time: Optional[float] = None

    last_packet_received_timestamp: Optional[float] = None
    """Epoch milliseconds when the last packet was received."""

    last_packet_sent_timestamp: Optional[float] = None
    """Epoch milliseconds when the last packet was sent."""

    local_candidate_address: Optional[str] = None

    local_candidate_id: Optional[str] = None

    local_candidate_network_type: Optional[str] = None

    local_candidate_port: Optional[float] = None

    local_candidate_protocol: Optional[str] = None

    local_candidate_related_address: Optional[str] = None

    local_candidate_related_port: Optional[float] = None

    local_candidate_type: Optional[str] = None

    local_candidate_url: Optional[str] = None

    nominated: Optional[bool] = None

    packets_discarded_on_send: Optional[float] = None

    packets_received: Optional[float] = None

    packets_sent: Optional[float] = None

    remote_candidate_address: Optional[str] = None

    remote_candidate_id: Optional[str] = None

    remote_candidate_port: Optional[float] = None

    remote_candidate_protocol: Optional[str] = None

    remote_candidate_type: Optional[str] = None

    remote_candidate_url: Optional[str] = None

    total_round_trip_time: Optional[float] = None


class DataParticipantPeerReportMetadataCandidatePairsProducingTransport(BaseModel):
    """ICE candidate pair statistics for a transport."""

    available_incoming_bitrate: Optional[float] = None

    available_outgoing_bitrate: Optional[float] = None

    bytes_discarded_on_send: Optional[float] = None

    bytes_received: Optional[float] = None

    bytes_sent: Optional[float] = None

    current_round_trip_time: Optional[float] = None

    last_packet_received_timestamp: Optional[float] = None
    """Epoch milliseconds when the last packet was received."""

    last_packet_sent_timestamp: Optional[float] = None
    """Epoch milliseconds when the last packet was sent."""

    local_candidate_address: Optional[str] = None

    local_candidate_id: Optional[str] = None

    local_candidate_network_type: Optional[str] = None

    local_candidate_port: Optional[float] = None

    local_candidate_protocol: Optional[str] = None

    local_candidate_related_address: Optional[str] = None

    local_candidate_related_port: Optional[float] = None

    local_candidate_type: Optional[str] = None

    local_candidate_url: Optional[str] = None

    nominated: Optional[bool] = None

    packets_discarded_on_send: Optional[float] = None

    packets_received: Optional[float] = None

    packets_sent: Optional[float] = None

    remote_candidate_address: Optional[str] = None

    remote_candidate_id: Optional[str] = None

    remote_candidate_port: Optional[float] = None

    remote_candidate_protocol: Optional[str] = None

    remote_candidate_type: Optional[str] = None

    remote_candidate_url: Optional[str] = None

    total_round_trip_time: Optional[float] = None


class DataParticipantPeerReportMetadataCandidatePairs(BaseModel):
    consuming_transport: Optional[List[DataParticipantPeerReportMetadataCandidatePairsConsumingTransport]] = None

    producing_transport: Optional[List[DataParticipantPeerReportMetadataCandidatePairsProducingTransport]] = None


class DataParticipantPeerReportMetadataDeviceInfo(BaseModel):
    cpus: Optional[float] = None

    is_mobile: Optional[bool] = None

    os: Optional[str] = None

    os_version: Optional[str] = None


class DataParticipantPeerReportMetadataEvent(BaseModel):
    """A timestamped event recorded during the participant's connection."""

    metadata: Optional[Dict[str, Union[str, float, bool, None]]] = None
    """Event-specific metadata.

    Keys vary per event; values are primitive scalars (string, number, boolean, or
    null).
    """

    name: Optional[str] = None
    """Name of the event."""

    timestamp: Optional[str] = None
    """Timestamp when the event occurred."""


class DataParticipantPeerReportMetadataIPInformationASN(BaseModel):
    asn: Optional[str] = None

    domain: Optional[str] = None

    name: Optional[str] = None

    route: Optional[str] = None

    type: Optional[str] = None


class DataParticipantPeerReportMetadataIPInformation(BaseModel):
    asn: Optional[DataParticipantPeerReportMetadataIPInformationASN] = None

    city: Optional[str] = None

    country: Optional[str] = None

    ipv4: Optional[str] = None

    org: Optional[str] = None

    region: Optional[str] = None

    timezone: Optional[str] = None


class DataParticipantPeerReportMetadataNativeMetadata(BaseModel):
    audio_encoder: Optional[str] = None

    video_encoder: Optional[str] = None


class DataParticipantPeerReportMetadataPcMetadata(BaseModel):
    effective_network_type: Optional[str] = None

    reflexive_connectivity: Optional[bool] = None

    relay_connectivity: Optional[bool] = None

    sdp: Optional[List[str]] = None

    timestamp: Optional[str] = None

    turn_connectivity: Optional[bool] = None


class DataParticipantPeerReportMetadataSelectedDeviceUpdateDevice(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataSelectedDeviceUpdate(BaseModel):
    device: Optional[DataParticipantPeerReportMetadataSelectedDeviceUpdateDevice] = None
    """A media device (camera, microphone, or speaker)."""

    timestamp: Optional[str] = None


class DataParticipantPeerReportMetadataSpeakerDevicesUpdateAdded(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataSpeakerDevicesUpdateRemoved(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataSpeakerDevicesUpdate(BaseModel):
    """A change to the set of available devices at a point in time."""

    added: Optional[List[DataParticipantPeerReportMetadataSpeakerDevicesUpdateAdded]] = None
    """Devices that became available."""

    removed: Optional[List[DataParticipantPeerReportMetadataSpeakerDevicesUpdateRemoved]] = None
    """Devices that became unavailable."""

    timestamp: Optional[str] = None
    """Timestamp of the device update."""


class DataParticipantPeerReportMetadataVideoDevicesUpdateAdded(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataVideoDevicesUpdateRemoved(BaseModel):
    """A media device (camera, microphone, or speaker)."""

    device_id: Optional[str] = None
    """ID of the device."""

    kind: Optional[str] = None
    """Kind of device, for example audioinput or videoinput."""

    label: Optional[str] = None
    """Human-readable label of the device."""


class DataParticipantPeerReportMetadataVideoDevicesUpdate(BaseModel):
    """A change to the set of available devices at a point in time."""

    added: Optional[List[DataParticipantPeerReportMetadataVideoDevicesUpdateAdded]] = None
    """Devices that became available."""

    removed: Optional[List[DataParticipantPeerReportMetadataVideoDevicesUpdateRemoved]] = None
    """Devices that became unavailable."""

    timestamp: Optional[str] = None
    """Timestamp of the device update."""


class DataParticipantPeerReportMetadata(BaseModel):
    """Connection and device metadata for the participant."""

    audio_devices_updates: Optional[List[DataParticipantPeerReportMetadataAudioDevicesUpdate]] = None

    browser_metadata: Optional[DataParticipantPeerReportMetadataBrowserMetadata] = None

    candidate_pairs: Optional[DataParticipantPeerReportMetadataCandidatePairs] = None

    device_info: Optional[DataParticipantPeerReportMetadataDeviceInfo] = None

    events: Optional[List[DataParticipantPeerReportMetadataEvent]] = None

    ip_information: Optional[DataParticipantPeerReportMetadataIPInformation] = None

    native_metadata: Optional[DataParticipantPeerReportMetadataNativeMetadata] = None

    pc_metadata: Optional[List[DataParticipantPeerReportMetadataPcMetadata]] = None

    room_view_type: Optional[str] = None

    sdk_name: Optional[str] = None

    sdk_type: Optional[str] = None

    sdk_version: Optional[str] = None

    selected_device_updates: Optional[List[DataParticipantPeerReportMetadataSelectedDeviceUpdate]] = None

    speaker_devices_updates: Optional[List[DataParticipantPeerReportMetadataSpeakerDevicesUpdate]] = None

    video_devices_updates: Optional[List[DataParticipantPeerReportMetadataVideoDevicesUpdate]] = None


class DataParticipantPeerReportQualityAudioConsumer(BaseModel):
    """Per-sample inbound (consumer) audio statistics."""

    bytes_received: Optional[float] = None

    concealment_events: Optional[float] = None

    consumer_id: Optional[str] = None

    jitter: Optional[float] = None

    jitter_buffer_delay: Optional[float] = None

    jitter_buffer_emitted_count: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_received: Optional[float] = None

    peer_id: Optional[str] = None

    producer_id: Optional[str] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityAudioConsumerCumulativeJitterBufferDelay(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityAudioConsumerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityAudioConsumerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityAudioConsumerCumulative(BaseModel):
    """Aggregated inbound (consumer) audio statistics for the session."""

    jitter_buffer_delay: Optional[DataParticipantPeerReportQualityAudioConsumerCumulativeJitterBufferDelay] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    packet_loss: Optional[DataParticipantPeerReportQualityAudioConsumerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityAudioConsumerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""


class DataParticipantPeerReportQualityAudioProducer(BaseModel):
    """Per-sample outbound (producer) audio statistics."""

    bytes_sent: Optional[float] = None

    jitter: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_sent: Optional[float] = None

    producer_id: Optional[str] = None

    rtt: Optional[float] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityAudioProducerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityAudioProducerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityAudioProducerCumulativeRTT(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityAudioProducerCumulative(BaseModel):
    """Aggregated outbound (producer) audio statistics for the session."""

    packet_loss: Optional[DataParticipantPeerReportQualityAudioProducerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityAudioProducerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""

    rtt: Optional[DataParticipantPeerReportQualityAudioProducerCumulativeRTT] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""


class DataParticipantPeerReportQualityScreenshareAudioConsumer(BaseModel):
    """Per-sample inbound (consumer) audio statistics."""

    bytes_received: Optional[float] = None

    concealment_events: Optional[float] = None

    consumer_id: Optional[str] = None

    jitter: Optional[float] = None

    jitter_buffer_delay: Optional[float] = None

    jitter_buffer_emitted_count: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_received: Optional[float] = None

    peer_id: Optional[str] = None

    producer_id: Optional[str] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativeJitterBufferDelay(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareAudioConsumerCumulative(BaseModel):
    """Aggregated inbound (consumer) audio statistics for the session."""

    jitter_buffer_delay: Optional[
        DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativeJitterBufferDelay
    ] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    packet_loss: Optional[DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityScreenshareAudioConsumerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""


class DataParticipantPeerReportQualityScreenshareAudioProducer(BaseModel):
    """Per-sample outbound (producer) audio statistics."""

    bytes_sent: Optional[float] = None

    jitter: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_sent: Optional[float] = None

    producer_id: Optional[str] = None

    rtt: Optional[float] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityScreenshareAudioProducerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareAudioProducerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareAudioProducerCumulativeRTT(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareAudioProducerCumulative(BaseModel):
    """Aggregated outbound (producer) audio statistics for the session."""

    packet_loss: Optional[DataParticipantPeerReportQualityScreenshareAudioProducerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityScreenshareAudioProducerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""

    rtt: Optional[DataParticipantPeerReportQualityScreenshareAudioProducerCumulativeRTT] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""


class DataParticipantPeerReportQualityScreenshareVideoConsumer(BaseModel):
    """Per-sample inbound (consumer) video statistics."""

    bytes_received: Optional[float] = None

    consumer_id: Optional[str] = None

    fir_count: Optional[float] = None

    frame_height: Optional[float] = None

    frame_width: Optional[float] = None

    frames_decoded: Optional[float] = None

    frames_dropped: Optional[float] = None

    frames_per_second: Optional[float] = None

    jitter: Optional[float] = None

    jitter_buffer_delay: Optional[float] = None

    jitter_buffer_emitted_count: Optional[float] = None

    key_frames_decoded: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_received: Optional[float] = None

    peer_id: Optional[str] = None

    producer_id: Optional[str] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeFramePerSecond(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeFrameWidth(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeIssues(BaseModel):
    lag_fraction: Optional[float] = None

    no_video_fraction: Optional[float] = None

    poor_resolution_fraction: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeJitterBufferDelay(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoConsumerCumulative(BaseModel):
    """Aggregated inbound (consumer) video statistics for the session."""

    frame_per_second: Optional[DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeFramePerSecond] = None
    """Distribution summary with average and percentiles."""

    frame_width: Optional[DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeFrameWidth] = None
    """Distribution summary with average and percentiles."""

    issues: Optional[DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeIssues] = None

    jitter_buffer_delay: Optional[
        DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeJitterBufferDelay
    ] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    key_frames_decoded_fraction: Optional[float] = None

    packet_loss: Optional[DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityScreenshareVideoConsumerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""


class DataParticipantPeerReportQualityScreenshareVideoProducerQualityLimitationDurations(BaseModel):
    bandwidth: Optional[float] = None

    cpu: Optional[float] = None

    none: Optional[float] = None

    other: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducer(BaseModel):
    """Per-sample outbound (producer) video statistics."""

    bytes_sent: Optional[float] = None

    fir_count: Optional[float] = None

    frame_height: Optional[float] = None

    frame_width: Optional[float] = None

    frames_encoded: Optional[float] = None

    frames_per_second: Optional[float] = None

    jitter: Optional[float] = None

    key_frames_encoded: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_sent: Optional[float] = None

    pli_count: Optional[float] = None

    producer_id: Optional[str] = None

    quality_limitation_durations: Optional[
        DataParticipantPeerReportQualityScreenshareVideoProducerQualityLimitationDurations
    ] = None

    quality_limitation_reason: Optional[Literal["cpu", "bandwidth", "none", "other"]] = None

    quality_limitation_resolution_changes: Optional[float] = None

    rtt: Optional[float] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeFramePerSecond(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeFrameWidth(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeIssues(BaseModel):
    bandwidth_quality_limitation_fraction: Optional[float] = None

    cpu_quality_limitation_fraction: Optional[float] = None

    no_video_fraction: Optional[float] = None

    poor_resolution_fraction: Optional[float] = None

    quality_limitation_fraction: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeRTT(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityScreenshareVideoProducerCumulative(BaseModel):
    """Aggregated outbound (producer) video statistics for the session."""

    frame_per_second: Optional[DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeFramePerSecond] = None
    """Distribution summary with average and percentiles."""

    frame_width: Optional[DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeFrameWidth] = None
    """Distribution summary with average and percentiles."""

    high_negative_feedback_fraction: Optional[float] = None

    issues: Optional[DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeIssues] = None

    key_frames_encoded_fraction: Optional[float] = None

    packet_loss: Optional[DataParticipantPeerReportQualityScreenshareVideoProducerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""

    rtt: Optional[DataParticipantPeerReportQualityScreenshareVideoProducerCumulativeRTT] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""


class DataParticipantPeerReportQualityVideoConsumer(BaseModel):
    """Per-sample inbound (consumer) video statistics."""

    bytes_received: Optional[float] = None

    consumer_id: Optional[str] = None

    fir_count: Optional[float] = None

    frame_height: Optional[float] = None

    frame_width: Optional[float] = None

    frames_decoded: Optional[float] = None

    frames_dropped: Optional[float] = None

    frames_per_second: Optional[float] = None

    jitter: Optional[float] = None

    jitter_buffer_delay: Optional[float] = None

    jitter_buffer_emitted_count: Optional[float] = None

    key_frames_decoded: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_received: Optional[float] = None

    peer_id: Optional[str] = None

    producer_id: Optional[str] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityVideoConsumerCumulativeFramePerSecond(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityVideoConsumerCumulativeFrameWidth(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityVideoConsumerCumulativeIssues(BaseModel):
    lag_fraction: Optional[float] = None

    no_video_fraction: Optional[float] = None

    poor_resolution_fraction: Optional[float] = None


class DataParticipantPeerReportQualityVideoConsumerCumulativeJitterBufferDelay(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityVideoConsumerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityVideoConsumerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityVideoConsumerCumulative(BaseModel):
    """Aggregated inbound (consumer) video statistics for the session."""

    frame_per_second: Optional[DataParticipantPeerReportQualityVideoConsumerCumulativeFramePerSecond] = None
    """Distribution summary with average and percentiles."""

    frame_width: Optional[DataParticipantPeerReportQualityVideoConsumerCumulativeFrameWidth] = None
    """Distribution summary with average and percentiles."""

    issues: Optional[DataParticipantPeerReportQualityVideoConsumerCumulativeIssues] = None

    jitter_buffer_delay: Optional[DataParticipantPeerReportQualityVideoConsumerCumulativeJitterBufferDelay] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    key_frames_decoded_fraction: Optional[float] = None

    packet_loss: Optional[DataParticipantPeerReportQualityVideoConsumerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityVideoConsumerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""


class DataParticipantPeerReportQualityVideoProducerQualityLimitationDurations(BaseModel):
    bandwidth: Optional[float] = None

    cpu: Optional[float] = None

    none: Optional[float] = None

    other: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducer(BaseModel):
    """Per-sample outbound (producer) video statistics."""

    bytes_sent: Optional[float] = None

    fir_count: Optional[float] = None

    frame_height: Optional[float] = None

    frame_width: Optional[float] = None

    frames_encoded: Optional[float] = None

    frames_per_second: Optional[float] = None

    jitter: Optional[float] = None

    key_frames_encoded: Optional[float] = None

    mid: Optional[str] = None

    mos_quality: Optional[float] = None

    packets_lost: Optional[float] = None

    packets_sent: Optional[float] = None

    pli_count: Optional[float] = None

    producer_id: Optional[str] = None

    quality_limitation_durations: Optional[DataParticipantPeerReportQualityVideoProducerQualityLimitationDurations] = (
        None
    )

    quality_limitation_reason: Optional[Literal["cpu", "bandwidth", "none", "other"]] = None

    quality_limitation_resolution_changes: Optional[float] = None

    rtt: Optional[float] = None

    ssrc: Optional[float] = None

    timestamp: Optional[str] = None


class DataParticipantPeerReportQualityVideoProducerCumulativeFramePerSecond(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducerCumulativeFrameWidth(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducerCumulativeIssues(BaseModel):
    bandwidth_quality_limitation_fraction: Optional[float] = None

    cpu_quality_limitation_fraction: Optional[float] = None

    no_video_fraction: Optional[float] = None

    poor_resolution_fraction: Optional[float] = None

    quality_limitation_fraction: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducerCumulativePacketLoss(BaseModel):
    """Cumulative packet loss distribution."""

    api_10_or_greater_event_fraction: Optional[float] = FieldInfo(alias="10_or_greater_event_fraction", default=None)

    api_25_or_greater_event_fraction: Optional[float] = FieldInfo(alias="25_or_greater_event_fraction", default=None)

    api_5_or_greater_event_fraction: Optional[float] = FieldInfo(alias="5_or_greater_event_fraction", default=None)

    api_50_or_greater_event_fraction: Optional[float] = FieldInfo(alias="50_or_greater_event_fraction", default=None)

    avg: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducerCumulativeQualityMos(BaseModel):
    """Distribution summary with average and percentiles."""

    avg: Optional[float] = None

    p50: Optional[float] = None

    p75: Optional[float] = None

    p90: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducerCumulativeRTT(BaseModel):
    """Cumulative latency distribution (milliseconds-based thresholds)."""

    api_100ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="100ms_or_greater_event_fraction", default=None
    )

    api_250ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="250ms_or_greater_event_fraction", default=None
    )

    api_500ms_or_greater_event_fraction: Optional[float] = FieldInfo(
        alias="500ms_or_greater_event_fraction", default=None
    )

    avg: Optional[float] = None


class DataParticipantPeerReportQualityVideoProducerCumulative(BaseModel):
    """Aggregated outbound (producer) video statistics for the session."""

    frame_per_second: Optional[DataParticipantPeerReportQualityVideoProducerCumulativeFramePerSecond] = None
    """Distribution summary with average and percentiles."""

    frame_width: Optional[DataParticipantPeerReportQualityVideoProducerCumulativeFrameWidth] = None
    """Distribution summary with average and percentiles."""

    high_negative_feedback_fraction: Optional[float] = None

    issues: Optional[DataParticipantPeerReportQualityVideoProducerCumulativeIssues] = None

    key_frames_encoded_fraction: Optional[float] = None

    packet_loss: Optional[DataParticipantPeerReportQualityVideoProducerCumulativePacketLoss] = None
    """Cumulative packet loss distribution."""

    quality_mos: Optional[DataParticipantPeerReportQualityVideoProducerCumulativeQualityMos] = None
    """Distribution summary with average and percentiles."""

    rtt: Optional[DataParticipantPeerReportQualityVideoProducerCumulativeRTT] = None
    """Cumulative latency distribution (milliseconds-based thresholds)."""


class DataParticipantPeerReportQuality(BaseModel):
    """Media quality statistics for the participant."""

    audio_consumer: Optional[List[DataParticipantPeerReportQualityAudioConsumer]] = None

    audio_consumer_cumulative: Optional[DataParticipantPeerReportQualityAudioConsumerCumulative] = None
    """Aggregated inbound (consumer) audio statistics for the session."""

    audio_producer: Optional[List[DataParticipantPeerReportQualityAudioProducer]] = None

    audio_producer_cumulative: Optional[DataParticipantPeerReportQualityAudioProducerCumulative] = None
    """Aggregated outbound (producer) audio statistics for the session."""

    screenshare_audio_consumer: Optional[List[DataParticipantPeerReportQualityScreenshareAudioConsumer]] = None

    screenshare_audio_consumer_cumulative: Optional[
        DataParticipantPeerReportQualityScreenshareAudioConsumerCumulative
    ] = None
    """Aggregated inbound (consumer) audio statistics for the session."""

    screenshare_audio_producer: Optional[List[DataParticipantPeerReportQualityScreenshareAudioProducer]] = None

    screenshare_audio_producer_cumulative: Optional[
        DataParticipantPeerReportQualityScreenshareAudioProducerCumulative
    ] = None
    """Aggregated outbound (producer) audio statistics for the session."""

    screenshare_video_consumer: Optional[List[DataParticipantPeerReportQualityScreenshareVideoConsumer]] = None

    screenshare_video_consumer_cumulative: Optional[
        DataParticipantPeerReportQualityScreenshareVideoConsumerCumulative
    ] = None
    """Aggregated inbound (consumer) video statistics for the session."""

    screenshare_video_producer: Optional[List[DataParticipantPeerReportQualityScreenshareVideoProducer]] = None

    screenshare_video_producer_cumulative: Optional[
        DataParticipantPeerReportQualityScreenshareVideoProducerCumulative
    ] = None
    """Aggregated outbound (producer) video statistics for the session."""

    video_consumer: Optional[List[DataParticipantPeerReportQualityVideoConsumer]] = None

    video_consumer_cumulative: Optional[DataParticipantPeerReportQualityVideoConsumerCumulative] = None
    """Aggregated inbound (consumer) video statistics for the session."""

    video_producer: Optional[List[DataParticipantPeerReportQualityVideoProducer]] = None

    video_producer_cumulative: Optional[DataParticipantPeerReportQualityVideoProducerCumulative] = None
    """Aggregated outbound (producer) video statistics for the session."""


class DataParticipantPeerReport(BaseModel):
    """Peer call statistics report."""

    metadata: Optional[DataParticipantPeerReportMetadata] = None
    """Connection and device metadata for the participant."""

    quality: Optional[DataParticipantPeerReportQuality] = None
    """Media quality statistics for the participant."""


class DataParticipant(BaseModel):
    id: Optional[str] = None
    """ID of the participant."""

    created_at: Optional[str] = None
    """timestamp when this participant was created."""

    custom_participant_id: Optional[str] = None
    """ID passed by client to create this participant."""

    display_name: Optional[str] = None
    """Display name of participant when joining the session."""

    duration: Optional[float] = None
    """number of minutes for which the participant was in the session."""

    joined_at: Optional[str] = None
    """timestamp at which participant joined the session."""

    left_at: Optional[str] = None
    """timestamp at which participant left the session."""

    peer_events: Optional[List[DataParticipantPeerEvent]] = None
    """Connection lifecycle events for the participant's peer."""

    peer_report: Optional[DataParticipantPeerReport] = None
    """Peer call statistics report."""

    role: Optional[str] = None
    """Name of the preset associated with the participant."""

    session_id: Optional[str] = None

    updated_at: Optional[str] = None
    """timestamp when this participant's data was last updated."""

    user_id: Optional[str] = None
    """User id for this participant."""


class Data(BaseModel):
    participant: Optional[DataParticipant] = None


class SessionGetParticipantDataFromPeerIDResponse(BaseModel):
    data: Optional[Data] = None

    success: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.dlp import custom_prompt_topic_create_params, custom_prompt_topic_update_params
from ....types.zero_trust.dlp.custom_prompt_topic import CustomPromptTopic

__all__ = ["CustomPromptTopicsResource", "AsyncCustomPromptTopicsResource"]


class CustomPromptTopicsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomPromptTopicsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CustomPromptTopicsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomPromptTopicsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CustomPromptTopicsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        enabled: bool,
        name: str,
        topic: str,
        description: Optional[str] | Omit = omit,
        profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPromptTopic]:
        """
        Creates a DLP custom prompt topic entry.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/dlp/custom_prompt_topics", account_id=account_id),
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "name": name,
                    "topic": topic,
                    "description": description,
                    "profile_id": profile_id,
                },
                custom_prompt_topic_create_params.CustomPromptTopicCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPromptTopic]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPromptTopic]], ResultWrapper[CustomPromptTopic]),
        )

    def update(
        self,
        entry_id: str,
        *,
        account_id: str,
        enabled: bool,
        name: str,
        topic: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPromptTopic]:
        """
        Updates a DLP custom prompt topic entry.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not entry_id:
            raise ValueError(f"Expected a non-empty value for `entry_id` but received {entry_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/dlp/custom_prompt_topics/{entry_id}", account_id=account_id, entry_id=entry_id
            ),
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "name": name,
                    "topic": topic,
                    "description": description,
                },
                custom_prompt_topic_update_params.CustomPromptTopicUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPromptTopic]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPromptTopic]], ResultWrapper[CustomPromptTopic]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CustomPromptTopic]:
        """
        Lists all DLP custom prompt topic entries in an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/custom_prompt_topics", account_id=account_id),
            page=SyncSinglePage[CustomPromptTopic],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CustomPromptTopic,
        )

    def delete(
        self,
        entry_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a DLP custom prompt topic entry.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not entry_id:
            raise ValueError(f"Expected a non-empty value for `entry_id` but received {entry_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/dlp/custom_prompt_topics/{entry_id}", account_id=account_id, entry_id=entry_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        entry_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPromptTopic]:
        """
        Fetches a DLP custom prompt topic entry by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not entry_id:
            raise ValueError(f"Expected a non-empty value for `entry_id` but received {entry_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/dlp/custom_prompt_topics/{entry_id}", account_id=account_id, entry_id=entry_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPromptTopic]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPromptTopic]], ResultWrapper[CustomPromptTopic]),
        )


class AsyncCustomPromptTopicsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomPromptTopicsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomPromptTopicsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomPromptTopicsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCustomPromptTopicsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        enabled: bool,
        name: str,
        topic: str,
        description: Optional[str] | Omit = omit,
        profile_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPromptTopic]:
        """
        Creates a DLP custom prompt topic entry.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/dlp/custom_prompt_topics", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "name": name,
                    "topic": topic,
                    "description": description,
                    "profile_id": profile_id,
                },
                custom_prompt_topic_create_params.CustomPromptTopicCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPromptTopic]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPromptTopic]], ResultWrapper[CustomPromptTopic]),
        )

    async def update(
        self,
        entry_id: str,
        *,
        account_id: str,
        enabled: bool,
        name: str,
        topic: str,
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPromptTopic]:
        """
        Updates a DLP custom prompt topic entry.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not entry_id:
            raise ValueError(f"Expected a non-empty value for `entry_id` but received {entry_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/dlp/custom_prompt_topics/{entry_id}", account_id=account_id, entry_id=entry_id
            ),
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "name": name,
                    "topic": topic,
                    "description": description,
                },
                custom_prompt_topic_update_params.CustomPromptTopicUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPromptTopic]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPromptTopic]], ResultWrapper[CustomPromptTopic]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomPromptTopic, AsyncSinglePage[CustomPromptTopic]]:
        """
        Lists all DLP custom prompt topic entries in an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/dlp/custom_prompt_topics", account_id=account_id),
            page=AsyncSinglePage[CustomPromptTopic],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CustomPromptTopic,
        )

    async def delete(
        self,
        entry_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a DLP custom prompt topic entry.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not entry_id:
            raise ValueError(f"Expected a non-empty value for `entry_id` but received {entry_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/dlp/custom_prompt_topics/{entry_id}", account_id=account_id, entry_id=entry_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        entry_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CustomPromptTopic]:
        """
        Fetches a DLP custom prompt topic entry by ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not entry_id:
            raise ValueError(f"Expected a non-empty value for `entry_id` but received {entry_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/dlp/custom_prompt_topics/{entry_id}", account_id=account_id, entry_id=entry_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CustomPromptTopic]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CustomPromptTopic]], ResultWrapper[CustomPromptTopic]),
        )


class CustomPromptTopicsResourceWithRawResponse:
    def __init__(self, custom_prompt_topics: CustomPromptTopicsResource) -> None:
        self._custom_prompt_topics = custom_prompt_topics

        self.create = to_raw_response_wrapper(
            custom_prompt_topics.create,
        )
        self.update = to_raw_response_wrapper(
            custom_prompt_topics.update,
        )
        self.list = to_raw_response_wrapper(
            custom_prompt_topics.list,
        )
        self.delete = to_raw_response_wrapper(
            custom_prompt_topics.delete,
        )
        self.get = to_raw_response_wrapper(
            custom_prompt_topics.get,
        )


class AsyncCustomPromptTopicsResourceWithRawResponse:
    def __init__(self, custom_prompt_topics: AsyncCustomPromptTopicsResource) -> None:
        self._custom_prompt_topics = custom_prompt_topics

        self.create = async_to_raw_response_wrapper(
            custom_prompt_topics.create,
        )
        self.update = async_to_raw_response_wrapper(
            custom_prompt_topics.update,
        )
        self.list = async_to_raw_response_wrapper(
            custom_prompt_topics.list,
        )
        self.delete = async_to_raw_response_wrapper(
            custom_prompt_topics.delete,
        )
        self.get = async_to_raw_response_wrapper(
            custom_prompt_topics.get,
        )


class CustomPromptTopicsResourceWithStreamingResponse:
    def __init__(self, custom_prompt_topics: CustomPromptTopicsResource) -> None:
        self._custom_prompt_topics = custom_prompt_topics

        self.create = to_streamed_response_wrapper(
            custom_prompt_topics.create,
        )
        self.update = to_streamed_response_wrapper(
            custom_prompt_topics.update,
        )
        self.list = to_streamed_response_wrapper(
            custom_prompt_topics.list,
        )
        self.delete = to_streamed_response_wrapper(
            custom_prompt_topics.delete,
        )
        self.get = to_streamed_response_wrapper(
            custom_prompt_topics.get,
        )


class AsyncCustomPromptTopicsResourceWithStreamingResponse:
    def __init__(self, custom_prompt_topics: AsyncCustomPromptTopicsResource) -> None:
        self._custom_prompt_topics = custom_prompt_topics

        self.create = async_to_streamed_response_wrapper(
            custom_prompt_topics.create,
        )
        self.update = async_to_streamed_response_wrapper(
            custom_prompt_topics.update,
        )
        self.list = async_to_streamed_response_wrapper(
            custom_prompt_topics.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            custom_prompt_topics.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            custom_prompt_topics.get,
        )
